"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const qs = require("qs");
const query_builder_constant_1 = require("./query-builder.constant");
class QueryStringBuilder {
    constructor() {
        this.queryFields = {
            limit: query_builder_constant_1.defaultOptions.perPage,
            page: query_builder_constant_1.defaultOptions.currentPage,
            sort: '',
            select: '',
            include: '',
            filter: '',
        };
    }
    filter(filter) {
        this.queryFields.filter = qs.stringify(filter);
        return this;
    }
    include(include) {
        this.queryFields.include = qs.stringify(include);
        return this;
    }
    select(select) {
        this.queryFields.select = select.join(',');
        return this;
    }
    sort(sort) {
        this.queryFields.sort = sort.join(',');
        return this;
    }
    paginate(paginate) {
        this.queryFields.limit = paginate.limit;
        this.queryFields.page = paginate.page;
        return this;
    }
    all({ filter, include, paginate, select, sort }) {
        filter && this.filter(filter);
        include && this.include(include);
        paginate && this.paginate(paginate);
        select && this.select(select);
        sort && this.sort(sort);
        return this;
    }
    build() {
        return qs.stringify(this.queryFields);
    }
}
exports.default = QueryStringBuilder;
//# sourceMappingURL=api-query-string-builder-index.js.map