"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const qs = require("qs");
const query_string_parser_1 = require("./query-string-parser");
const query_builder_constant_1 = require("./query-builder.constant");
class ApiQueryBuilder {
    constructor(modelEntity, entity, query, validStringFilter = query_builder_constant_1.defaultPrismaStringFilter) {
        this.builder = {
            filter: {},
            paginated: { currentPage: 1, pages: 1 },
            select: {},
            sort: {},
            paginate: { skip: query_builder_constant_1.skip, take: query_builder_constant_1.take }
        };
        this.builderObj = {
            skip: query_builder_constant_1.skip,
            take: query_builder_constant_1.take,
            where: {},
            select: {},
            orderBy: {}
        };
        this.modelEntity = modelEntity;
        this.query = query !== null && query !== void 0 ? query : query_builder_constant_1.defaultQueryFields;
        this.entity = entity;
        this.validStringFilter = validStringFilter;
        console.log("\n \nCONSTRUCTOR", { querySelect: query.select, defaultQueryFieldsSelect: query_builder_constant_1.defaultQueryFields.select, builder: this.builder });
    }
    filter() {
        var _a, _b, _c;
        const filterObj = (0, query_string_parser_1.parseObject)(qs.parse((_b = (_a = this === null || this === void 0 ? void 0 : this.query) === null || _a === void 0 ? void 0 : _a.filter) !== null && _b !== void 0 ? _b : '', { comma: true }));
        const validFilterObj = this.filterRecursive(filterObj);
        console.log("\n \n FILTER", { filterObj, queryFilter: (_c = this === null || this === void 0 ? void 0 : this.query) === null || _c === void 0 ? void 0 : _c.filter, validFilterObj });
        this.builder.filter = validFilterObj;
        return this;
    }
    sort() {
        var _a, _b, _c;
        const sort = _.intersectionWith(((_b = (_a = this === null || this === void 0 ? void 0 : this.query) === null || _a === void 0 ? void 0 : _a.sort) !== null && _b !== void 0 ? _b : query_builder_constant_1.defaultQueryFields.sort).split(','), [
            ...this.entity.sortableColumns,
            ...this.entity.sortableColumns.map((col) => `-${col}`),
        ]);
        this.builder.sort = sort === null || sort === void 0 ? void 0 : sort.map((s) => {
            const key = (s === null || s === void 0 ? void 0 : s.startsWith('-')) ? s === null || s === void 0 ? void 0 : s.slice(1, s === null || s === void 0 ? void 0 : s.length) : s;
            return { [key]: (s === null || s === void 0 ? void 0 : s.startsWith('-')) ? 'desc' : 'asc' };
        });
        console.log("\n \nSORT", {
            sort,
            'defaultQueryFields.sort': query_builder_constant_1.defaultQueryFields.sort,
            'this?.query?.sort': (_c = this === null || this === void 0 ? void 0 : this.query) === null || _c === void 0 ? void 0 : _c.sort,
            'this.entity.sortableColumns': this.entity.sortableColumns,
            'this.builder.sort': this.builder.sort,
        });
        return this;
    }
    select() {
        var _a;
        const selectObj = qs.parse(`query=${((_a = this === null || this === void 0 ? void 0 : this.query) === null || _a === void 0 ? void 0 : _a.select) || 'true'}`, {
            comma: true,
        });
        this.builder.select = this.arrayToBoolObject(this.entity.selectableColumns, this.toArray(selectObj === null || selectObj === void 0 ? void 0 : selectObj.query));
        console.log("\n \nSELECT", {
            builderSelect: this.builder.select,
            selectableColumns: this.entity.selectableColumns,
            selectObjQuery: this.toArray(selectObj === null || selectObj === void 0 ? void 0 : selectObj.query),
        });
        return this;
    }
    populate() {
        var _a, _b, _c;
        const includeObj = qs.parse((_b = (_a = this === null || this === void 0 ? void 0 : this.query) === null || _a === void 0 ? void 0 : _a.include) !== null && _b !== void 0 ? _b : '', { comma: true });
        if ((0, query_builder_constant_1.isObjectEmpty)(includeObj))
            return this;
        (_c = this.entity.allowedRelationShips) === null || _c === void 0 ? void 0 : _c.forEach((col) => {
            var _a;
            const selectables = this.entity.selectableRelationShips[col];
            const includeQueryParam = this.toArray((_a = includeObj[col]) !== null && _a !== void 0 ? _a : []);
            const filteredIncludeObj = this.arrayToBoolObject(selectables, includeQueryParam);
            if (!this.builder.select)
                this.builder.select = {};
            this.builder.select[col] = {
                select: includeQueryParam[0] == 'true'
                    ? this.arrayToBoolObject(selectables, selectables)
                    : filteredIncludeObj !== null && filteredIncludeObj !== void 0 ? filteredIncludeObj : undefined,
            };
            console.log("\n \nPOPULATE LOOP", {
                selectables,
                includeQueryParam,
                filteredIncludeObj,
                ['builderSelect' + col]: this.builder.select[col],
            });
        });
        console.log("\n \nPOPULATE", {
            includeObj,
        });
        return this;
    }
    paginate() {
        var _a, _b, _c, _d;
        const page = ((_b = (_a = this.query) === null || _a === void 0 ? void 0 : _a.page) !== null && _b !== void 0 ? _b : query_builder_constant_1.defaultOptions.currentPage) * 1 || 1;
        const limit = ((_d = (_c = this.query) === null || _c === void 0 ? void 0 : _c.limit) !== null && _d !== void 0 ? _d : query_builder_constant_1.defaultOptions.perPage) * 1 || 50;
        const skip = (page - 1) * limit;
        this.builder.paginate = { skip, take: limit };
        console.log("\n \n PAGINATE", { page, limit, skip, 'this.builder.paginate': this.builder.paginate });
        return this;
    }
    build() {
        const { filter, paginate, select, sort } = this.builder;
        this.builderObj = Object.assign({ where: filter, select, orderBy: sort }, paginate);
        console.log("\n \nBuilder Object", Object.assign({}, this.builderObj));
        return this;
    }
    all() {
        this.sort();
        this.filter();
        this.select();
        this.populate();
        this.paginate();
        return this;
    }
    async many() {
        var _a, _b, _c;
        const data = await this.modelEntity.findMany(this.builderObj);
        const totalRecords = await this.modelEntity.count({
            where: this.builderObj.where,
        });
        const paginated = {
            currentPage: (_b = (_a = this.query) === null || _a === void 0 ? void 0 : _a.page) !== null && _b !== void 0 ? _b : query_builder_constant_1.defaultQueryFields.page * 1,
            perPage: ((_c = this.query.limit) !== null && _c !== void 0 ? _c : query_builder_constant_1.defaultQueryFields.limit) * 1 || (data === null || data === void 0 ? void 0 : data.length),
            totalPages: Math.ceil(totalRecords / this.builderObj.take),
            totalRecords,
        };
        if (paginated.currentPage > 1 &&
            paginated.currentPage > paginated.totalPages)
            return 'NotFoundException';
        return { data, paginated };
    }
    async findMany(filter) {
        var _a, _b, _c;
        if (!(filter === null || filter === void 0 ? void 0 : filter.where))
            return 'Invalid Filter Parameter';
        const data = await this.modelEntity.findMany(filter);
        const totalRecords = await this.modelEntity.count({
            where: filter === null || filter === void 0 ? void 0 : filter.where,
        });
        const paginated = {
            currentPage: (_b = (_a = this.query) === null || _a === void 0 ? void 0 : _a.page) !== null && _b !== void 0 ? _b : query_builder_constant_1.defaultQueryFields.page * 1,
            perPage: ((_c = this.query.limit) !== null && _c !== void 0 ? _c : query_builder_constant_1.defaultQueryFields.limit) * 1 || (data === null || data === void 0 ? void 0 : data.length),
            totalPages: Math.ceil(totalRecords / this.builderObj.take),
            totalRecords,
        };
        if (paginated.currentPage > 1 &&
            paginated.currentPage > paginated.totalPages)
            return 'NotFoundException';
        return { data, paginated };
    }
    async one(filter) {
        if (!(filter === null || filter === void 0 ? void 0 : filter.where))
            return 'Invalid Filter Parameter';
        const _a = this.builderObj, { where, skip, take } = _a, obj = __rest(_a, ["where", "skip", "take"]);
        console.log({ filter });
        return this.modelEntity.findFirst(Object.assign({ where: Object.assign(Object.assign({}, filter === null || filter === void 0 ? void 0 : filter.where), where) }, obj));
    }
    toArray(val) {
        if (val == undefined || val == null)
            return [];
        if (Array.isArray(val))
            return val.filter((col) => col != undefined);
        return [val];
    }
    arrayToBoolObject(selectables, array) {
        if (array[0] == 'true')
            return this.arrayToBoolObject(selectables, selectables);
        array = _.intersectionWith(array, selectables);
        const obj = {};
        array === null || array === void 0 ? void 0 : array.forEach((element) => {
            obj[element] = true;
        });
        return (0, query_builder_constant_1.isObjectEmpty)(obj) ? {} : obj;
    }
    filterRecursive(filterObj) {
        var _a, _b, _c, _d;
        const validFilterObj = _.pick(filterObj, [
            ...this.entity.filterableColumns,
            'AND',
            'NOT',
            'OR',
            ...this.entity.filterableReltionship,
        ]);
        for (const key in validFilterObj) {
            const filterValue = validFilterObj[key];
            if (typeof filterValue != 'string' && !Array.isArray(filterValue)) {
                const trimmedFilterValue = _.pick(filterValue, [
                    ...this.validStringFilter,
                    ...this.entity.filterableReltionshipColumns,
                ]);
                validFilterObj[key] = (0, query_builder_constant_1.isObjectEmpty)(trimmedFilterValue)
                    ? undefined
                    : trimmedFilterValue;
                if ((_a = validFilterObj[key]) === null || _a === void 0 ? void 0 : _a.in)
                    validFilterObj[key].in = this.toArray((_b = validFilterObj[key]) === null || _b === void 0 ? void 0 : _b.in.split(','));
                if ((_c = validFilterObj[key]) === null || _c === void 0 ? void 0 : _c.notIn)
                    validFilterObj[key].notIn = this.toArray((_d = validFilterObj[key]) === null || _d === void 0 ? void 0 : _d.notIn.split(','));
            }
        }
        return validFilterObj;
    }
}
exports.default = ApiQueryBuilder;
//# sourceMappingURL=query-builder.js.map