"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.queryParseMiddleware = exports.parseValue = exports.parseBoolean = exports.parseNumber = exports.parseArray = exports.parseObject = exports.isArray = exports.isBoolean = exports.isNumber = exports.isObject = void 0;
const isObject = (val) => {
    return val.constructor === Object;
};
exports.isObject = isObject;
const isNumber = (val) => {
    return !isNaN(parseFloat(val)) && isFinite(val);
};
exports.isNumber = isNumber;
const isBoolean = (val) => {
    return val === 'false' || val === 'true';
};
exports.isBoolean = isBoolean;
const isArray = (val) => {
    return Array.isArray(val);
};
exports.isArray = isArray;
const parseObject = (obj) => {
    var result = {};
    var key, val;
    for (key in obj) {
        val = (0, exports.parseValue)(obj[key]);
        if (val !== null)
            result = { [key]: val };
    }
    return result;
};
exports.parseObject = parseObject;
const parseArray = (arr) => {
    var result = [];
    for (var i = 0; i < arr.length; i++) {
        result[i] = (0, exports.parseValue)(arr[i]);
    }
    return result;
};
exports.parseArray = parseArray;
const parseNumber = (val) => {
    return Number(val);
};
exports.parseNumber = parseNumber;
const parseBoolean = (val) => {
    return val === 'true';
};
exports.parseBoolean = parseBoolean;
const parseValue = (val) => {
    if (typeof val === 'undefined' || val === '') {
        return null;
    }
    else if ((0, exports.isBoolean)(val)) {
        return (0, exports.parseBoolean)(val);
    }
    else if ((0, exports.isArray)(val)) {
        return (0, exports.parseArray)(val);
    }
    else if ((0, exports.isObject)(val)) {
        return (0, exports.parseObject)(val);
    }
    else if ((0, exports.isNumber)(val)) {
        return (0, exports.parseNumber)(val);
    }
    else {
        return val;
    }
};
exports.parseValue = parseValue;
const queryParseMiddleware = () => {
    return (req, _res, next) => {
        req.query = (0, exports.parseObject)(req.query);
        next();
    };
};
exports.queryParseMiddleware = queryParseMiddleware;
//# sourceMappingURL=query-string-parser.js.map